/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "adiabaticPerfectFluid.H"
#include "IOstreams.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class Specie>
Foam::adiabaticPerfectFluid<Specie>::adiabaticPerfectFluid(Istream& is)
:
    Specie(is),
    p0_(readScalar(is)),
    rho0_(readScalar(is)),
    gamma_(readScalar(is)),
    B_(readScalar(is))
{
    is.check
    (
        "adiabaticPerfectFluid<Specie>::adiabaticPerfectFluid(Istream& is)"
    );
}


template<class Specie>
Foam::adiabaticPerfectFluid<Specie>::adiabaticPerfectFluid
(
    const dictionary& dict
)
:
    Specie(dict),
    p0_(readScalar(dict.subDict("equationOfState").lookup("p0"))),
    rho0_(readScalar(dict.subDict("equationOfState").lookup("rho0"))),
    gamma_(readScalar(dict.subDict("equationOfState").lookup("gamma"))),
    B_(readScalar(dict.subDict("equationOfState").lookup("B")))
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template<class Specie>
void Foam::adiabaticPerfectFluid<Specie>::write(Ostream& os) const
{
    Specie::write(os);

    dictionary dict("equationOfState");
    dict.add("p0", p0_);
    dict.add("rho0", rho0_);
    dict.add("gamma", gamma_);
    dict.add("B", B_);

    os  << indent << dict.dictName() << dict;
}


// * * * * * * * * * * * * * * * Ostream Operator  * * * * * * * * * * * * * //

template<class Specie>
Foam::Ostream& Foam::operator<<
(
    Ostream& os,
    const adiabaticPerfectFluid<Specie>& pf
)
{
    os  << static_cast<const Specie&>(pf)
        << token::SPACE << pf.R_
        << token::SPACE << pf.rho0_
        << token::SPACE << pf.gamma_
        << token::SPACE << pf.B_;

    os.check
    (
        "Ostream& operator<<(Ostream&, const adiabaticPerfectFluid<Specie>&)"
    );

    return os;
}


// ************************************************************************* //
